<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * This page is the protocol page with summarising stytistics.
 *
 * @package    mod
 * @subpackage stupla
 * @copyright  2012 Rüdiger Krauße
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

header("Expires: 0");

include 'protlib.php';
include 'data_load.php';
include 'prot_util.php';

define("STUPLA_DS_TEXTS", 1);
define("STUPLA_DS_MEDIA", 2);
define("STUPLA_DS_EX", 4);
define("STUPLA_DS_EX_SPECIAL", 8);
define("STUPLA_DS_USE_EX_NAMES", 16);
define("STUPLA_DS_FIRSTGLANCE", 32);

$PAGE->set_url('/mod/stupla/prot/prot_statistic.php', array('id' => $cm->id));
$PAGE->navbar->ignore_active();

$ds = 255;

if ( isset($_SESSION['stupla_ds']) )
	$ds = $_SESSION['stupla_ds'];
else
	$_SESSION['stupla_ds'] = $ds;

if ( optional_param('setDS', '', PARAM_ALPHA) == 'true' ) {
	$ds = 0;
	$ds |= optional_param('dsTexts', 0, PARAM_INT);
	$ds |= optional_param('dsMedia', 0, PARAM_INT);
	$ds |= optional_param('dsEx', 0, PARAM_INT);
	$ds |= optional_param('dsExSpecial', 0, PARAM_INT);
	$ds |= optional_param('dsUseExNames', 0, PARAM_INT);
	$ds |= optional_param('dsFirstGlance', 0, PARAM_INT);
	$_SESSION['stupla_ds'] = $ds;
}

// generate basic table data

$table = new html_table();

$table->name = get_string('statistic', 'stupla');
$table->head = array(
	get_string('code', 'stupla'),
	get_string('nlogin', 'stupla'),
	get_string('time', 'stupla')
);
$table->align = array('left', 'right', 'right');
$table->data = array();
$table->ordertype = array('','',''); //,'a','','','','p','p');

// subuserdata
if ( $stupla->flags & STUPLA_USESUBUSERS ) {
	stupla_expand_subuserdata($stupla);
	foreach($stupla->sudata as $field) 	{
		array_push($table->head, $field['name']);
		array_push($table->align, 'left');
		array_push($table->ordertype, '');
	}
}

// header: texts
if ( $ds & STUPLA_DS_TEXTS ) {
	for ($i = 0; $i < count($Name); $i++) {
		array_push($table->head, 'tText'.($i+1));
		array_push($table->align, 'right');
		array_push($table->ordertype, '');
	}
}

// header: media
if ( $ds & STUPLA_DS_MEDIA ) {
	for ($topic = 0; $topic < count($Name); $topic++)
//		if ( isset($Media[$topic]) )
			for ($type = 0; $type < count($MediaTitle); $type++)
				if ( isset($Media[$type][$topic]) && $type != $M_AUFGABE )
					for ($i = 0; $i < count($Media[$type][$topic]); $i++) {
						$medid = ($topic+1).(count($Media[$type][$topic]) == 1 ? '' : '_'.($i+1));
						$medName = ($ds & STUPLA_DS_USE_EX_NAMES) ? PureFileName(GetMedia($type, $topic, $i, 0)) : $MediaTitleShort[$type].$medid;
						array_push($table->head, 't'.$medName);
						array_push($table->align, 'right');
						array_push($table->ordertype, '');
					}
}

// header: exercises
if ( $ds & (STUPLA_DS_EX | STUPLA_DS_EX_SPECIAL) ) {
	$SpecialEx = array();
	$SpecialEx_AddHeader = array(
		'NONEGAME0' => 'SpecialEx_AddHeader_NONEGAME0',
		'NONEGAME1' => 'SpecialEx_AddHeader_NONEGAME1',
		'NONEGAME2' => 'SpecialEx_AddHeader_NONEGAME2',
		'NONEGAME3' => 'SpecialEx_AddHeader_NONEGAME3'
	);
	$SpecialEx_AddData = array(
		'NONEGAME0' => 'SpecialEx_AddData_NONEGAME0',
		'NONEGAME1' => 'SpecialEx_AddData_NONEGAME1',
		'NONEGAME2' => 'SpecialEx_AddData_NONEGAME2',
		'NONEGAME3' => 'SpecialEx_AddData_NONEGAME3'
	);
	$sError = "";
	MakeExerciseList($stupla, GetSessions($stupla));
	for ($topic = 0; $topic < count($Name); $topic++)
		if ( array_key_exists($topic, $Media[$M_AUFGABE]) )
		{
			for ($i = 0; $i < count($Media[$M_AUFGABE][$topic]); $i++)
			{
				$exid = ($topic+1).(count($Media[$M_AUFGABE][$topic]) == 1 ? '' : '_'.($i+1));
				$exName = ($ds & STUPLA_DS_USE_EX_NAMES) ? PureFileName(GetMedia($M_AUFGABE, $topic, $i, 0)) : "ex".$exid;
				// entries
				// filename: $Media[$M_AUFGABE][$topic][$i][0]
				// special exercise types
				if ( isset($ExEntry[$topic][$i][0][0][0][1]) && FALSE !== strpos($ExEntry[$topic][$i][0][0][0][1], '[NONEGAME1]') && ($ds & STUPLA_DS_EX_SPECIAL) )
				{
					$SpecialEx[$exid] = 'NONEGAME1'; // EF-Version of Essay-Launcher (WPAL)
					$SpecialEx_AddHeader['NONEGAME1']($table, $exName);
				}
				else if ( isset($ExEntry[$topic][$i][0][0][0][1]) && FALSE !== strpos($ExEntry[$topic][$i][0][0][0][1], '[NONEGAME2]') && ($ds & STUPLA_DS_EX_SPECIAL) )
				{
					$SpecialEx[$exid] = 'NONEGAME2'; // Essay Launcher (WPAL)
					$SpecialEx_AddHeader['NONEGAME2']($table, $exName);
				}
				else if ( isset($ExEntry[$topic][$i][0][0][0][1]) && FALSE !== strpos($ExEntry[$topic][$i][0][0][0][1], '[NONEGAME0]') && ($ds & STUPLA_DS_EX_SPECIAL) )
				{
					$SpecialEx[$exid] = 'NONEGAME0'; // IW (WPAL)
					$SpecialEx_AddHeader['NONEGAME0']($table, $exName);
				}
				else if ( isset($ExEntry[$topic][$i][0][0][0][1]) && FALSE !== strpos($ExEntry[$topic][$i][0][0][0][1], '[NONEGAME3]') && ($ds & STUPLA_DS_EX_SPECIAL) )
				{
					$SpecialEx[$exid] = 'NONEGAME3'; // READING (WPAL)
					$SpecialEx_AddHeader['NONEGAME3']($table, $exName);
				}
				else if ( $ds & STUPLA_DS_EX ) {
					// time
					AddHeader($table, 't_'.$exName, 'right', 'n');
					// grade
					AddHeader($table, 'grade_'.$exName, 'right', 'n');
					// percent
					AddHeader($table, 'pct_'.$exName, 'right', 'n');
					for ($j = 0; $j < count($ExEntry[$topic][$i][0]) || $j < count($ExEntry[$topic][$i][1]); $j++)
					{
						// entries
						AddHeader($table, 'ent_'.$exName.'_'.($j+1));
					}
				}
			}
		}
}

// fill with data
$sessions = GetSessions($stupla);

$curSessionName = '';

foreach ($sessions as $session)
{
	ClearHist();
	ReadHist($session, $stupla, $ds & STUPLA_DS_FIRSTGLANCE);
	$curSessionName = $session->displayname;
	$d = array($format == "showashtml"
			? '<a href="javascript:top.control.SelectHist(\''.$session->id.'\')" target="control">'.$session->displayname.'</a>'
			: $session->displayname,
			$LoginCount,
			$TotalTime);
	// subuserdata
	if ( $stupla->flags & STUPLA_USESUBUSERS ) {
		$subdat = isset($session->data) ? explode("\x01", $session->data) : array();
		for ($i = 0; $i < count($stupla->sudata); $i++)
			array_push($d, isset($subdat[$i]) ? $subdat[$i] : '');
	}

	// time text visited
	if ( $ds & STUPLA_DS_TEXTS ) {
		for ($i = 0; $i < count($Name); $i++) {
			array_push($d, $TextVisited[$i]);
		}
	}

	// time media
	if ( $ds & STUPLA_DS_MEDIA )
		for ($topic = 0; $topic < count($Name); $topic++)
			for ($type = 0; $type < count($MediaTitle); $type++)
				if ( isset($Media[$type][$topic]) && $type != $M_AUFGABE )
					for ($i = 0; $i < count($Media[$type][$topic]); $i++) {
						array_push($d, $Media[$type][$topic][$i][3] != -3 ? $Media[$type][$topic][$i][3] : '');
					}
	// data: exercises
	if ( $ds & (STUPLA_DS_EX | STUPLA_DS_EX_SPECIAL) ) {
		for ($topic = 0; $topic < count($Name); $topic++)
			if ( array_key_exists($topic, $Media[$M_AUFGABE]) )
			{
				for ($i = 0; $i < count($Media[$M_AUFGABE][$topic]); $i++)
				{
					$exid = ($topic+1).(count($Media[$M_AUFGABE][$topic]) == 1 ? '' : '_'.($i+1));
					if ( isset($SpecialEx[$exid]) )
					{
						if ( $ds & STUPLA_DS_EX_SPECIAL )
							$SpecialEx_AddData[$SpecialEx[$exid]]($session, $stupla, $d);
					}
					else if ( $ds & STUPLA_DS_EX )
					{
						$mediaEx = $Media[$M_AUFGABE][$topic][$i];
						// time
						array_push($d, isset($mediaEx[5]) ? $mediaEx[5]: '');
						// grade
						array_push($d, isset($mediaEx[3]) && $mediaEx[3] != -3 ? ($mediaEx[3] >= 98 ? 1 : 0) : '');
						// percent
						array_push($d, isset($mediaEx[3]) && $mediaEx[3] != -3 ? $mediaEx[3] : '');
						// entries
						// filename: $Media[$M_AUFGABE][$topic][$i][0]
						$mediaData = isset($mediaEx[4]) ? explode('|', $mediaEx[4]) : array();
						for ($j = 0; $j < count($ExEntry[$topic][$i][0]) || $j < count($ExEntry[$topic][$i][1]); $j++)
						{
							// entries
							$sData = isset($mediaData[$j]) ? $mediaData[$j] : '';
							if ( substr($sData, 0, 4) == '[mc]' )
								$sData = substr($sData, 4)*1;
							$sData = str_replace('_', ' ', $sData);
							$sData = str_replace('////', $format == "showashtml" ? '<br/>' : "\r\n", $sData);
							array_push($d, $sData);
						}
					}
				}
			}
	}


	// append line to table
	$table->data[] = $d;
}

$tables = array($table);

if ( $format == "showashtml" ) {
	print_header();
?>
	<form>
		<input type="hidden" name="n" value="<?php echo $stupla->id ?>"/>
		<input type="hidden" name="setDS" value="true"/>
		<input type="checkbox" name="dsTexts" value="<?php echo STUPLA_DS_TEXTS ?>" <?php if ( $ds & STUPLA_DS_TEXTS ) echo ' checked="checked"'; ?>/><?php print_string('display_texts', 'stupla') ?>
		<input type="checkbox" name="dsMedia" value="<?php echo STUPLA_DS_MEDIA ?>" <?php if ( $ds & STUPLA_DS_MEDIA ) echo ' checked="checked"'; ?>/><?php print_string('display_media', 'stupla') ?>
		<input type="checkbox" name="dsEx" value="<?php echo STUPLA_DS_EX ?>" <?php if ( $ds & STUPLA_DS_EX ) echo ' checked="checked"'; ?>/><?php print_string('display_exercises', 'stupla') ?>
		<input type="checkbox" name="dsExSpecial" value="<?php echo STUPLA_DS_EX_SPECIAL ?>" <?php if ( $ds & STUPLA_DS_EX_SPECIAL ) echo ' checked="checked"'; ?>/><?php print_string('display_special_exercises', 'stupla') ?>
		<input type="checkbox" name="dsUseExNames" value="<?php echo STUPLA_DS_USE_EX_NAMES ?>" <?php if ( $ds & STUPLA_DS_USE_EX_NAMES ) echo ' checked="checked"'; ?>/><?php print_string('use_media_exercise_file_names', 'stupla') ?>
		<input type="checkbox" name="dsFirstGlance" value="<?php echo STUPLA_DS_FIRSTGLANCE ?>" <?php if ( $ds & STUPLA_DS_FIRSTGLANCE ) echo ' checked="checked"'; ?>/><?php print_string('use_exercises_first_glance', 'stupla') ?>
		<input type="submit" name="submit" value="<?php print_string('apply_changes', 'stupla') ?>"/>
	</form>
<?php
	print_tables_html($tables);
    echo $sError;
}
else if ( $format == "downloadascsv" )
	print_tables_csv("StatisticList", $tables);
else if ( $format == "downloadasexcel" )
	print_tables_xls("StatisticList", $tables);

// ------------------------------------------------------------------
// Helper functions
// ------------------------------------------------------------------

function AddHeader(&$table, $head, $align = 'left', $ordertype = '')
{
	array_push($table->head, $head);
	array_push($table->align, $align);
	array_push($table->ordertype, $ordertype);

}

// ------------------------------------------------------------------
// Special exercises
// ------------------------------------------------------------------


// Writing
function SpecialEx_AddHeader_NONEGAME0(&$table, $exName)
{
	// time
	AddHeader($table, 't_'.$exName, 'right', 'n');
	// grade
	AddHeader($table, 'grade_'.$exName, 'right', 'n');
	// percent
	AddHeader($table, 'pct_'.$exName, 'right', 'n');
	// solved blocks - whether the exercise was solved more than one time
	AddHeader($table, 'n_used_'.$exName, 'right', 'n');
	for ($i = 1; $i <= 4; $i++)
	{
		AddHeader($table, 't_promt_'.$i, 'right', 'n');
		AddHeader($table, 'cont_'.$i, 'left'. '');
		AddHeader($table, 'n_help'.$i, 'right', 'n');
		AddHeader($table, 't_help'.$i, 'right', 'n');
	}
}

function SpecialEx_AddHeader_NONEGAME1(&$table, $exName)
{
	// time
	AddHeader($table, 't_'.$exName, 'right', 'n');
	// grade
	AddHeader($table, 'grade_'.$exName, 'right', 'n');
	// percent
	AddHeader($table, 'pct_'.$exName, 'right', 'n');
	// solved blocks - whether the exercise was solved more than one time
	AddHeader($table, 'n_used_'.$exName, 'right', 'n');
	for ($i = 1; $i < 5; $i++)
	{
		AddHeader($table, 'trial_'.$i, 'right'. 'n');
		AddHeader($table, 'left_'.$i, 'right'. 'n');
		AddHeader($table, 'n_prompt_task'.$i, 'right'. 'n');
		for ($j = 1; $j < 6; $j++)
		{
			AddHeader($table, 't_prompt'.$j.'_task'.$i, 'n');
			AddHeader($table, 'n_trial_prompt'.$j.'_task'.$i, 'n');
			AddHeader($table, 'prompt'.$j.'_task'.$i, 'n');
			AddHeader($table, 'n_help'.$j.'_task'.$i, 'n');
			AddHeader($table, 't_help'.$j.'_task'.$i, 'n');
		}
	}
}

function SpecialEx_AddHeader_NONEGAME2(&$table, $exName)
{
	// time
	AddHeader($table, 't_'.$exName, 'right', 'n');
	// grade
	AddHeader($table, 'grade_'.$exName, 'right', 'n');
	// percent
	AddHeader($table, 'pct_'.$exName, 'right', 'n');
	// solved blocks - whether the exercise was solved more than one time
	AddHeader($table, 'n_used_'.$exName, 'right', 'n');
	for ($i = 1; $i < 5; $i++)
	{
		AddHeader($table, 'trial_'.$i, 'right'. 'n');
		AddHeader($table, 'left_'.$i, 'right'. 'n');
		AddHeader($table, 'n_prompt_task'.$i, 'right'. 'n');
		for ($j = 1; $j < 6; $j++)
		{
			AddHeader($table, 't_prompt'.$j.'_task'.$i, 'n');
			AddHeader($table, 'n_trial_prompt'.$j.'_task'.$i, 'n');
			AddHeader($table, 'prompt'.$j.'_task'.$i, 'n');
			AddHeader($table, 'n_help'.$j.'_task'.$i, 'n');
			AddHeader($table, 't_help'.$j.'_task'.$i, 'n');
		}
	}
}

// Reading
function SpecialEx_AddHeader_NONEGAME3(&$table, $exName)
{
	// time
	AddHeader($table, 't_'.$exName, 'right', 'n');
	// grade
	AddHeader($table, 'grade_'.$exName, 'right', 'n');
	// percent
	AddHeader($table, 'pct_'.$exName, 'right', 'n');
	// solved blocks - whether the exercise was solved more than one time
	AddHeader($table, 'n_used_'.$exName, 'right', 'n');
	for ($i = 1; $i < 5; $i++)
	{
		for ($j = 1; $j < 6; $j++)
		{
			AddHeader($table, 't_prompt'.$j.'_task'.$i, 'n');
			AddHeader($table, 'n_help'.$j.'_task'.$i, 'n');
			AddHeader($table, 't_help'.$j.'_task'.$i, 'n');
		}
	}
}

$nSolved = 0;

function ReadNONEGAME($session, $stupla, $nonegameType) {
	global $nSolved, $curSessionName;
	$entry = array();
	$reading = FALSE;
	$nSolved = 0;
	$prefix = '[NONEGAME'.$nonegameType.']';
	if ($records = $DB->get_records_select("stupla_action", "stupla='$stupla->id' AND session='$session->id'", "starttime ASC", "*"))
		foreach ($records as $a) {
			if ( isset($a->data) ) {
				if ( substr($a->data, 0, 23) == $prefix.'[1,0] submit' ) {
					$keep = (count($entry) > 0) ? TRUE : FALSE;
					for ($i = 0; $i < count($entry); $i++)
						if ( strpos($entry[$i]->data, '[1,0] showed') === FALSE && strpos($entry[$i]->data, '[1,1] showed') === FALSE )
							$keep = FALSE;
					if ( !$keep ) {
						$entry = array();
						$reading = true;
						$nSolved++;
					}
				}
				if ( $reading && (substr($a->data, 0, 11) == $prefix) ) {
					array_push($entry, $a);
					if ( $nSolved == 0 )
						$nSolved = 1;
				}
				else
					$reading = false;
			}
			else
				$reading = true;
		}

	// reorder strange entries
	for ($i = 0; $i < count($entry)-1; $i++) {
		if ( preg_match('/^\[NONEGAME\d\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $entry[$i]->data, $b1) &&
			preg_match('/^\[NONEGAME\d\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $entry[$i+1]->data, $b2)) {
				if ( $b1[2] != -1 && $b2[2] != -1 && $b1[1] == $b2[1] && $b1[2] > $b2[2] ) {
					// echo "inversion[", $curSessionName, "][", $entry[$i]->data, "][", $entry[$i+1]->data, "][Delta: ", $entry[$i+1]->start - $entry[$i]->start,"]", '<br>'; // debud
					// swap
					$swap = $entry[$i];
					$entry[$i] = $entry[$i+1];
					$entry[$i+1] = $swap;
				}
		}
	}
	return $entry;
}

function SpecialEx_AddData_NONEGAME0($session, $stupla, &$d)
{
	global $nSolved;
	// find entries
	$entry = ReadNONEGAME($session, $stupla, '0');

	// collect data
	$res = array();
	for ($i = 0; $i < 4; $i++)
		$res[$i] = array(0, FALSE, 0, 0); // time, contents, n_hints, t_hints

	// process entries
	$lastTime = 0;
	$lastWasHint = FALSE;
	foreach ($entry as $a) {
		// get position and type
		if ( preg_match('/^\[NONEGAME0\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $a->data, $b) ) {
			$iRaw = $b[1]-1;
			$iFine = $b[2]+0;
			$type = $b[3];
			$data = $b[4];
			if ( $iRaw == 0 )
				$iFine -= 1;
			// ignore very first screen
			if  ( $iFine < 0 )
				continue;

			if ( $type == 'showed' ) {
				if ( $res[$iRaw][0] == 0 )
					$res[$iRaw][0] = $a->starttime;
				if ( $lastWasHint ) {
					$res[$iRaw][3] += $a->duration;
					$lastWasHint = FALSE;
				}
			}
			else if ( $type == 'confirm' ) {
				if ( $res[$iRaw][0] == 0 )
					$res[$iRaw][0] = $a->starttime;
				$res[$iRaw][0] = $a->starttime - $res[$iRaw][0];
				$res[$iRaw][1] = $data;
			}
			else if ( $type == 'submit' ) {
				if ( $res[$iRaw][0] == 0 )
					$res[$iRaw][0] = $a->starttime;
				$res[$iRaw][0] = $a->starttime - $res[$iRaw][0];
			}
			else if ( $type == 'nextExercise' ) {
			}
			else if ( $type == 'overview' ) {
			}
			else if ( $type == 'returnMenue' ) {
			}
			else if ( $type == 'hint' ) {
				$res[$iRaw][2]++;
				$lastWasHint = TRUE;
			}
			else
				echo "Unexpected type in NONEGAME0: ", $type;
		}
		else
			echo "Unknown NONEGAME0: ", $a->data;
		$lastTime = $a->starttime;
	}

	// write out
	array_push($d, $entry[count($entry)-1]->starttime-$entry[0]->starttime); // time
	array_push($d, 'TODO'); // grade
	array_push($d, 'TODO'); // pct
	array_push($d, $nSolved); // nSolved
	for ($i = 0; $i < 4; $i++)
	{
		if ( $res[$i][1] !== FALSE ) {
			array_push($d, $res[$i][0]);
			array_push($d, str_replace('_', ' ', $res[$i][1]));
			array_push($d, $res[$i][2]);
			array_push($d, $res[$i][3]);
		}
		else {
			array_push($d, '');
			array_push($d, '');
			array_push($d, '');
			array_push($d, '');
		}
	}
}

function SpecialEx_AddData_NONEGAME1($session, $stupla, &$d)
{
	global $curSessionName, $nSolved, $format;
	// find entries
	$entry = ReadNONEGAME($session, $stupla, '1');

	// collect data
	$resRaw = array();
	for ($i = 0; $i < 4; $i++)
		$resRaw[$i] = array(0, 10, 0, ''); // trial(= number of corect solved paragraphs), left (of the 10 trial), n_promt task, skip-log (how the next promt was entered)
	$resFine = array();

	// reorder strange entries
	for ($i = 0; $i < count($entry)-1; $i++) {
		if ( preg_match('/^\[NONEGAME1\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $entry[$i]->data, $b1) &&
			preg_match('/^\[NONEGAME1\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $entry[$i+1]->data, $b2)) {
				if ( $b1[2] != -1 && $b2[2] != -1 && $b1[1] == $b2[1] && $b1[2] > $b2[2] ) {
					//echo "inversion[", $curSessionName, "][", $entry[$i]->data, "][", $entry[$i+1]->data, "][Delta: ", $entry[$i+1]->start - $entry[$i]->start,"]", '<br>';
					// swap
					$swap = $entry[$i];
					$entry[$i] = $entry[$i+1];
					$entry[$i+1] = $swap;
				}
		}
	}

	EnrichNONEGAME1($entry);

	// process entries
	$lastRaw = -1; $lastFine = -1;
	$lastWasHint = FALSE;
	$lastEndTime = 0;
	foreach ($entry as $a) {
		// get position and type
		if ( preg_match('/^\[NONEGAME1\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $a->data, $b) ) {
			$iRaw = $b[1]-1;
			$iFine = $b[2]+0;
			$type = $b[3];
			$data = $b[4];
			// correction for first raw:
			if ( $iRaw == 0 )
				$iFine -= 2;

			if ( $type == 'showed' ) {
				if ( !isset($resFine[$iRaw]) )
					$resFine[$iRaw] = array();
				if ( $iFine >= 0 && !isset($resFine[$iRaw][$iFine]) ) {
					$resFine[$iRaw][$iFine] = array($a->starttime, 0, '', 0, 0); // time/duration, n_trials, promt_contents, n_hints, t_hints
					$resRaw[$iRaw][2]++;
					$resRaw[$iRaw][3] .= 's'.$iFine;
				}
				if ( $lastWasHint ) {
					$resFine[$iRaw][$iFine][4] += $a->duration;
					$lastWasHint = FALSE;
				}
			}
			else if ( ($type == 'confirm') || ($type == '*confirm') || ($type == 'systerror') ) {
				// secure for missing showed event
				if ( !isset($resFine[$iRaw]) )
					$resFine[$iRaw] = array();
				if ( $iFine >= 0 && !isset($resFine[$iRaw][$iFine]) ) {
					$resFine[$iRaw][$iFine] = array($lastEndTime, 0, '', 0, 0); // time/duration, n_trials, promt_contents, n_hints, t_hints
					$resRaw[$iRaw][2]++;
					$resRaw[$iRaw][3] .= 'c'.$iFine;
				}
				// do the regular stuff
				if ( $a->result == 100 ) {
					$resRaw[$iRaw][0]++;
				}
				$resRaw[$iRaw][1]--;
				//echo "[[", $iRaw, ", ", $iFine, "]]";
				$resFine[$iRaw][$iFine][1]++;
				// content
				$sCont = '';
				$c = explode('|', str_replace(' ', '', str_replace('[mc]', '', MCPart($a))));
				$sCont .= $c[$iFine*2].','.$c[$iFine*2+1];
				$resFine[$iRaw][$iFine][2] .= ($resFine[$iRaw][$iFine][2] == '' ? '' : '/').$sCont;
			}
			else if ( $type == 'hint' || $type == '*hint') {
				// secure for missing showed event
				if ( !isset($resFine[$iRaw]) )
					$resFine[$iRaw] = array();
				if ( $iFine >= 0 && !isset($resFine[$iRaw][$iFine]) ) {
					$resFine[$iRaw][$iFine] = array($lastEndTime, 0, '', 0, 0); // time/duration, n_trials, promt_contents, n_hints, t_hints
					$resRaw[$iRaw][2]++;
					$resRaw[$iRaw][3] .= 'h'.$iFine;
				}
				// do the regular stuff
				$resFine[$iRaw][$iFine][3]++;
				$lastWasHint = TRUE;
			}
			else if ( $type == 'submit' ) {
			}
			else if ( $type == 'overview' ) {
			}
			else if ( $type == 'nextExercise' ) {
				$iRaw++;
				$iFine = 0;
				$resFine[$iRaw] = array();
				$resFine[$iRaw][$iFine] = array($a->starttime, 0, '', 0, 0); // time/duration, n_trials, ?, ?
				$resRaw[$iRaw][2]++;
				$resRaw[$iRaw][3] .= 'n'.$iFine;
			}
			else if ( $type == 'returnMenue' ) {
			}
			else
				echo "Unexpected type in NONEGAME1[", $curSessionName, "]: ", $type;

			if ( $lastRaw >= 0 && $lastFine >= 0 && ($iRaw != $lastRaw || $iFine != $lastFine) )
				if ( $resFine[$lastRaw][$lastFine][0] > 1000 )
					$resFine[$lastRaw][$lastFine][0] = $a->starttime - $resFine[$lastRaw][$lastFine][0]; // strftime("%a %d %b %y, %H:%M:%S", $resFine[$lastRaw][$lastFine][0]).' '.strftime("%a %d %b %y, %H:%M:%S", $a->starttime);
			$lastRaw = $iRaw;
			$lastFine = $iFine;
			$lastEndTime = $a->starttime + $a->duration; // used for missing start events
		}
		else
			echo "Unknown NONEGAME1[", $curSessionName, "]: ", $a->data;
	}

	// write out
	array_push($d, $entry[count($entry)-1]->starttime-$entry[0]->starttime); // time
	array_push($d, 'TODO'); // grade
	array_push($d, 'TODO'); // pct
	array_push($d, $nSolved); // nSolved
	for ($i = 0; $i < 4; $i++)
	{
		if ( $resRaw[$i][2] > 0 ) {
			array_push($d, $resRaw[$i][0]);
			array_push($d, $resRaw[$i][1].($format == "showashtml" && $resRaw[$i][1] > 0 && $resRaw[$i][0] < 5 ? '(ERROR?)' : ''));
			array_push($d, $resRaw[$i][2].($format == "showashtml" && $resRaw[$i][0] > $resRaw[$i][2] ? '(ERROR?)' : ''));
		}
		else {
			array_push($d, '');
			array_push($d, '');
			array_push($d, '');
		}
		for ($j = 0; $j < 5; $j++)
		{
			if ( isset($resFine[$i][$j]) ) {
				array_push($d, $resFine[$i][$j][0]);
				array_push($d, $resFine[$i][$j][1]);
				array_push($d, $resFine[$i][$j][2]);
				array_push($d, $resFine[$i][$j][3]);
				array_push($d, $resFine[$i][$j][4]);
			}
			else {
				array_push($d, '');
				array_push($d, '');
				array_push($d, '');
				array_push($d, '');
				array_push($d, '');
			}
		}
	}
}

function SpecialEx_AddData_NONEGAME2($session, $stupla, &$d)
{
	global $nSolved;

	// find entries
	$entry = ReadNONEGAME($session, $stupla, '2');

	// collect data
	$resRaw = array();
	for ($i = 0; $i < 4; $i++)
		$resRaw[$i] = array(0, 10, 0); // trial(= number of corect solved paragraphs), left (of the 10 trial), n_promt task
	$resFine = array();

	// process entries
	$lastRaw = -1; $lastFine = -1;
	$lastWasHint = FALSE;
	foreach ($entry as $a) {
		// get position and type
		if ( preg_match('/^\[NONEGAME2\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $a->data, $b) ) {
			$iRaw = $b[1]-1;
			$iFine = $b[2];
			$type = $b[3];
			$data = $b[4];
			// correction for first raw:
			if ( $iRaw == 0 )
				$iFine -= 2;

			if ( $type == 'showed' ) {
				if ( !isset($resFine[$iRaw]) )
					$resFine[$iRaw] = array();
				if ( !isset($resFine[$iRaw][$iFine]) ) {
					$resFine[$iRaw][$iFine] = array($a->starttime, 0, '', 0, 0); // time/duration, n_trials, promt_contents, n_hints, t_hints
					$resRaw[$iRaw][2]++;
				}
				if ( $lastWasHint ) {
					$resFine[$iRaw][$iFine][4] += $a->duration;
					$lastWasHint = FALSE;
				}
			}
			else if ( $type == 'confirm' ) {
				if ( $a->result == 100 ) {
					$resRaw[$iRaw][0]++;
				}
				$resRaw[$iRaw][1]--;
				//echo "[[", $iRaw, ", ", $iFine, "]]";
				$resFine[$iRaw][$iFine][1]++;
				// content
				$sCont = '';
				if ( preg_match('/^TaskID:(\d+)/', $data, $b) ) {
					 if ( $resFine[$iRaw][$iFine][2] == '' )
					 	$sCont .= 'TaskID:'.$b[1].': ';
				}
				if ( preg_match('/\s+(\S+\s*\|\s*\d+)\s*$/', $data, $b) ) {
					$sCont .= $b[1];
				}
				if ($sCont == '')
					$sCont = "Error: unparseable";
				$resFine[$iRaw][$iFine][2] .= ($resFine[$iRaw][$iFine][2] == '' ? '' : '/').$sCont;
			}
			else if ( $type == 'hint' ) {
				$resFine[$iRaw][$iFine][3]++;
				$lastWasHint = TRUE;
			}
			else if ( $type == 'submit' ) {
			}
			else if ( $type == 'overview' ) {
			}
			else if ( $type == 'nextExercise' ) {
				$iRaw++;
				$iFine = 0;
				$resFine[$iRaw] = array();
				$resFine[$iRaw][$iFine] = array($a->starttime, 0, '', 0, 0); // time/duration, n_trials, ?, ?
			}
			else if ( $type == 'returnMenue' ) {
			}
			else
				echo "Unexpected type in NONEGAME2: ", $type;

			if ( $lastRaw >= 0 && $lastFine >= 0 && ($iRaw != $lastRaw || $iFine != $lastFine) )
				if ( $resFine[$lastRaw][$lastFine][0] > 1000 )
					$resFine[$lastRaw][$lastFine][0] = $a->starttime - $resFine[$lastRaw][$lastFine][0];
			$lastRaw = $iRaw;
			$lastFine = $iFine;
		}
		else
			echo "Unknown NONEGAME2: ", $a->data;
	}

	// write out
	array_push($d, /*strftime("%a %d %b %y, %H:%M:%S", $entry[0]->starttime)*/$entry[count($entry)-1]->starttime-$entry[0]->starttime); // time
	array_push($d, 'TODO'); // grade
	array_push($d, 'TODO'); // pct
	array_push($d, $nSolved); // nSolved
	for ($i = 0; $i < 4; $i++)
	{
		if ( $resRaw[$i][2] > 0 ) {
			array_push($d, $resRaw[$i][0]);
			array_push($d, $resRaw[$i][1]);
			array_push($d, $resRaw[$i][2] > 5 ? 5 : $resRaw[$i][2]); // could be 1 to much (additional "showed" at the end)
		}
		else {
			array_push($d, '');
			array_push($d, '');
			array_push($d, '');
		}
		for ($j = 0; $j < 5; $j++)
		{
			if ( isset($resFine[$i][$j]) ) {
				array_push($d, $resFine[$i][$j][0]);
				array_push($d, $resFine[$i][$j][1]);
				array_push($d, $resFine[$i][$j][2]);
				array_push($d, $resFine[$i][$j][3]);
				array_push($d, $resFine[$i][$j][4]);
			}
			else {
				array_push($d, '');
				array_push($d, '');
				array_push($d, '');
				array_push($d, '');
				array_push($d, '');
			}
		}
	}
}

function SpecialEx_AddData_NONEGAME3($session, $stupla, &$d)
{
	global $curSessionName, $nSolved, $format;
	// find entries
	$entry = ReadNONEGAME($session, $stupla, '3');

	// collect data
	$resRaw = array();
	for ($i = 0; $i < 4; $i++)
		$resRaw[$i] = array(0, 10, 0, ''); // trial(= number of corect solved paragraphs), left (of the 10 trial), n_promt task, skip-log (how the next promt was entered)
	$resFine = array();

	// process entries
	$lastRaw = -1; $lastFine = -1;
	$lastWasHint = FALSE;
	$lastEndTime = 0;
	foreach ($entry as $a) {
		// get position and type
		if ( preg_match('/^\[NONEGAME3\]\[(\d+),([\d\-]+)\] (\S+)\s*(.*)$/', $a->data, $b) ) {
			$iRaw = $b[1]-1;
			$iFine = $b[2]+0;
			$type = $b[3];
			$data = $b[4];
			// correction for first raw:
			if ( $iRaw == 0 )
				$iFine -= 1;

			if ( $type == 'showed' ) {
				if ( !isset($resFine[$iRaw]) )
					$resFine[$iRaw] = array();
				if ( $iFine >= 0 && !isset($resFine[$iRaw][$iFine]) ) {
					$resFine[$iRaw][$iFine] = array($a->starttime, 0, '', 0, 0); // time/duration, n_trials, promt_contents, n_hints, t_hints
					$resRaw[$iRaw][2]++;
					$resRaw[$iRaw][3] .= 's'.$iFine;
				}
				if ( $lastWasHint ) {
					$resFine[$iRaw][$iFine][4] += $a->duration;
					$lastWasHint = FALSE;
				}
			}
			else if ( ($type == 'confirm') || ($type == '*confirm') || ($type == 'systerror') ) {
				// secure for missing showed event
				if ( !isset($resFine[$iRaw]) )
					$resFine[$iRaw] = array();
				if ( $iFine >= 0 && !isset($resFine[$iRaw][$iFine]) ) {
					$resFine[$iRaw][$iFine] = array($lastEndTime, 0, '', 0, 0); // time/duration, n_trials, promt_contents, n_hints, t_hints
					$resRaw[$iRaw][2]++;
					$resRaw[$iRaw][3] .= 'c'.$iFine;
				}
				// do the regular stuff
				if ( $a->result == 100 ) {
					$resRaw[$iRaw][0]++;
				}
				$resRaw[$iRaw][1]--;
				//echo "[[", $iRaw, ", ", $iFine, "]]";
				$resFine[$iRaw][$iFine][1]++;
				// content
				$sCont = '';
				$c = explode('|', str_replace(' ', '', str_replace('[mc]', '', MCPart($a))));
				$sCont .= $c[$iFine*2].','.$c[$iFine*2+1];
				$resFine[$iRaw][$iFine][2] .= ($resFine[$iRaw][$iFine][2] == '' ? '' : '/').$sCont;
			}
			else if ( $type == 'hint' || $type == '*hint') {
				// secure for missing showed event
				if ( !isset($resFine[$iRaw]) )
					$resFine[$iRaw] = array();
				if ( $iFine >= 0 && !isset($resFine[$iRaw][$iFine]) ) {
					$resFine[$iRaw][$iFine] = array($lastEndTime, 0, '', 0, 0); // time/duration, n_trials, promt_contents, n_hints, t_hints
					$resRaw[$iRaw][2]++;
					$resRaw[$iRaw][3] .= 'h'.$iFine;
				}
				// do the regular stuff
				$resFine[$iRaw][$iFine][3]++;
				$lastWasHint = TRUE;
			}
			else if ( $type == 'submit' ) {
			}
			else if ( $type == 'overview' ) {
			}
			else if ( $type == 'nextExercise' ) {
				$iRaw++;
				$iFine = 0;
				$resFine[$iRaw] = array();
				$resFine[$iRaw][$iFine] = array($a->starttime, 0, '', 0, 0); // time/duration, n_trials, ?, ?
				$resRaw[$iRaw][2]++;
				$resRaw[$iRaw][3] .= 'n'.$iFine;
			}
			else if ( $type == 'returnMenue' ) {
			}
			else
				echo "Unexpected type in NONEGAME3[", $curSessionName, "]: ", $type;

			if ( $lastRaw >= 0 && $lastFine >= 0 && ($iRaw != $lastRaw || $iFine != $lastFine) )
				if ( $resFine[$lastRaw][$lastFine][0] > 1000 )
					$resFine[$lastRaw][$lastFine][0] = $a->starttime - $resFine[$lastRaw][$lastFine][0]; // strftime("%a %d %b %y, %H:%M:%S", $resFine[$lastRaw][$lastFine][0]).' '.strftime("%a %d %b %y, %H:%M:%S", $a->starttime);
			$lastRaw = $iRaw;
			$lastFine = $iFine;
			$lastEndTime = $a->starttime + $a->duration; // used for missing start events
		}
		else
			echo "Unknown NONEGAME3[", $curSessionName, "]: ", $a->data;
	}

	// write out
	array_push($d, $entry[count($entry)-1]->starttime-$entry[0]->starttime); // time
	array_push($d, 'TODO'); // grade
	array_push($d, 'TODO'); // pct
	array_push($d, $nSolved); // nSolved
	for ($i = 0; $i < 4; $i++)
	{
		for ($j = 0; $j < 5; $j++)
		{
			if ( isset($resFine[$i][$j]) ) {
				array_push($d, $resFine[$i][$j][0]);
				array_push($d, $resFine[$i][$j][3]);
				array_push($d, $resFine[$i][$j][4]);
			}
			else {
				array_push($d, '');
				array_push($d, '');
				array_push($d, '');
			}
		}
	}
}

?>