<?PHP //$Id: restorelib.php,v 1.1 2007/11/23 23:31:33 krausze Exp $

require_once ("$CFG->dirroot/mod/stupla/lib.php");

function stupla_restore_mods($mod, $restore) {
    //This function restores a single stupla activity

    // This function is called by "restore_create_modules" (in "backup/restorelib.php")
    // which is called by "backup/restore_execute.html" (included by "backup/restore.php")
    // $mod is an object
    //     id           : id field in 'modtype' table
    //     modtype      : 'stupla'
    // $restore is an object
    //     backup_unique_code : xxxxxxxxxx
    //     file         : '/full/path/to/backupfile.zip'
    //     mods         : an array of $modinfo's (see below)
    //     restoreto    : 0=existing course (replace), 1=existing course (append), 2=new course
    //     users        : 0=all, 1=course, 2=none
    //     logs         : 0=no, 1=yes
    //     user_files   : 0=no, 1=yes
    //     course_files : 0=no, 1=yes
    //     course_id    : id of course into which data is to be restored
    //     deleting     : true if 'restoreto'==0, otherwise false
    //     original_wwwroot : 'http://your.server.com/moodle'
    // $modinfo is an array
    //    'modname'    : array( 'restore'=> 0=no 1=yes, 'userinfo' => 0=no 1=yes)

    global $CFG;
    $status = true;
    $more_restore = '';
    // get course module data this stupla activity
    $data = backup_getid($restore->backup_unique_code, 'stupla', $mod->id);
    if ($data) {
        // $data is an object
        //    backup_code => xxxxxxxxxx,
        //    table_name  => 'stupla',
        //    old_id      => xxx,
        //    new_id      => NULL,
        //    info        => xml tree array of info backed up for this stupla activity
        $xml = &$data->info['MOD']['#'];
        $table = 'stupla';
        $foreign_keys = array('course' => $restore->course_id);
        // print a message after each stupla is backed up
        if (!defined('RESTORE_SILENTLY')) {
            $more_restore .= 'print "<li>".get_string("modulename", "stupla")." &quot;".format_string(stripslashes($record->name),true)."&quot;</li>";';
        }
        $more_restore .= 'backup_flush(300);';
        if (function_exists('restore_userdata_selected')) {
            // Moodle >= 1.6
            $restore_userdata_selected = restore_userdata_selected($restore, 'stupla', $mod->id);
        } else {
            // Moodle <= 1.5
            $restore_userdata_selected = $restore->mods['stupla']->userinfo;
        }
        if ($restore_userdata_selected) {
            $more_restore .= '$status = stupla_restore_simple_table($restore, $status, $xml, $record, "stupla_session", "SESSION");';
        	$more_restore .= '$status = stupla_restore_simple_table($restore, $status, $xml, $record, "stupla_action", "ACTION");';
            $more_restore .= '$status = stupla_restore_simple_table($restore, $status, $xml, $record, "stupla_ex", "EX");';
            $more_restore .= '$status = stupla_restore_simple_table($restore, $status, $xml, $record, "stupla_plan", "PLAN");';
            $more_restore .= '$status = stupla_restore_simple_table($restore, $status, $xml, $record, "stupla_sheet", "SHEET");';
        }
        // if necessary, adjust stupla date/time fields and write to restorelog
        if ($restore->course_startdateoffset) {
            restore_log_date_changes('stupla', $restore, $xml, array('TIMEOPEN', 'TIMECLOSE', 'TIMECREATED', 'TIMEMODIFIED'));
        }

        $status = stupla_restore_records(
            $restore, $status, $xml, $table, $foreign_keys, $more_restore
        );
    }
    return $status;
}

function stupla_restore_records(&$restore, $status, &$xml, $table, $foreign_keys, $more_restore='', $records_TAG='', $record_TAG='', $secondary_key='') {
// general purpose function to restore a group of records
    // $restore : (see "stupla_restore_mods" above)
    // $xml : an XML tree (or sub-tree)
    // $records_TAG : (optional) the name of an XML tag which starts a block of records
    //    If no $records_TAG is specified, $xml is assumed to be a block of records
    // $record_TAG  : (optional) the name of an XML tag which starts a single record
    //    If no $record_TAG is specified, the block of records is assumed to be a single record
    // other parameters are explained in "stupla_restore_record" below

    $i = 0; // index for $records_TAG
    do {
        unset($xml_records);
        if ($records_TAG) {
            if (isset($xml[$records_TAG][$i]['#'])) {
                $xml_records = &$xml[$records_TAG][$i]['#'];
            }
        } else {
            if ($i==0) {
                $xml_records = &$xml;
            }
        }
        if (isset($xml_records)) {
            $ii = 0; // index for $record_TAG
            do {
                unset($xml_record);
                if ($record_TAG) {
                    if (isset($xml_records[$record_TAG][$ii]['#'])) {
                        $xml_record = &$xml_records[$record_TAG][$ii]['#'];
                    }
                } else {
                    if ($ii==0) {
                        $xml_record = &$xml_records;
                    }
                }
                if (isset($xml_record)) {
                    $status = stupla_restore_record(
                        $restore, $status, $xml_record, $table, $foreign_keys, $more_restore, $secondary_key
                    );
                }
                $ii++;
            } while ($status && isset($xml_record));
        }
        $i++;
    } while ($status && isset($xml_records));
    return $status;
}

function stupla_restore_record(&$restore, $status, &$xml, $table, $foreign_keys, $more_restore, $secondary_key) {
// general purpose function to restore a single record
    // $restore : (see "stupla_restore_mods" above)
    // $status : current status of backup (true or false)
    // $xml    : XML tree of current record
    // $table  : name of Moodle database table to restore to
    // $foreign_keys : array of foreign keys, if any, specifed as $key=>$value
    //    $key   : the name of a field in the current $record
    //    $value : if $value is numeric, then $record->$key is set to $value.
    //        Otherwise $value is assumed to be a table name and $record->$key
    //        is treated as a comma separated list of ids in that table
    // $more_restore : optional PHP code to be eval(uated) for each record
    // $secondary_key :
    //    the name of the secondary key field, if any, in the current $record.
    //    If this field is specified, then the current record will only be added
    //    if the $record->$secondarykey value does not already exist in $table

    // maintain a cache of info on table columns
    static $table_columns = array();
    if (empty($table_columns[$table])) {
        global $CFG, $db;
        $table_columns[$table] = $db->MetaColumns("$CFG->prefix$table");
    }

    // get values for fields in this record
    $record = new stdClass();
    $TAGS = array_keys($xml);
    foreach ($TAGS as $TAG) {
        $value = $xml[$TAG][0]['#'];
        if (is_string($value)) {
            $tag = strtolower($TAG);
            $record->$tag = backup_todb(stupla_safe_uncode($value));
        }
    }

    // update foreign keys, if any
    $ok = true;
    foreach ($foreign_keys as $key=>$value) {
        if (is_numeric($value)) {
            $record->$key = $value;
        } else {
            $key_table = $value;
            $new_ids = array();
            if (isset($record->$key)) {
                $old_ids = explode(',', $record->$key);
                foreach ($old_ids as $old_id) {
                    if (empty($old_id)) {
                        // do nothing
                    } else {
                        $key_record = backup_getid($restore->backup_unique_code, $key_table, $old_id);
                        if ($key_record) {
                            $new_ids[] = $key_record->new_id;
                        } else {
                            // foreign key could not be updated
                            if (!defined('RESTORE_SILENTLY')) {
                                print "<ul><li><b>Warning:</b><br>Foreign key could not be updated:<br>";
                                print "'$key_table' record (old id=$old_id) is missing from backup data<br>";
                                print "'$table' record ";
                                if (isset($record->id)) {
                                    print "(old id=$record->id) ";
                                }
                                print "was not restored</li></ul>";
                            }
                            $ok = false;
                        }
                    }
                }
            }
            $record->$key = implode(',', $new_ids);
        }
    }

    // check all "not null" fields have been set
    foreach ($table_columns[$table] as $column) {
        if ($column->not_null) {
            $name = $column->name;
            if ($name<>'id' && !isset($record->$name)) {
                if (isset($column->default_value)) {
                    $default = $column->default_value;
                } else {
                    if (preg_match('/int|decimal|double|float|time|year/i', $column->type)) {
                        $default = 0;
                    } else {
                        $default = '';
                    }
                }
                $record->$name = $default;
            }
        }
    }

    // check everything is OK so far
    if ($ok) {
        // store old record id, if necessary
        if (isset($record->id)) {
            $record->old_id = $record->id;
            unset($record->id);
        }
        // if there is a secondary key field  ...
        if ($secondary_key) {
            // check to see if a record with the same value already exists
            $key_record = get_record($table, $secondary_key, $record->$secondary_key);
            if ($key_record) {
                // set new record id from already existing record
                $record->id = $key_record->id;
            }
        }
        if (empty($record->id)) {
            // add the $record (and get new id)
            $record->id = insert_record($table, $record);
        }
        // check $record was added (or found)
        if (is_numeric($record->id)) {
            // if there was an old id, save a mapping to the new id
            if (isset($record->old_id)) {
                backup_putid($restore->backup_unique_code, $table, $record->old_id, $record->id);
            }
        } else {
            // failed to add (or find) $record
            if (!defined('RESTORE_SILENTLY')) {
                print "<ul><li>Record could not be added: table=$table</li></ul>";
            }
            $status = false;
        }
        // restore related records, if required
        if ($more_restore) {
            eval($more_restore);
        }
    }
    return $status;
}

function stupla_restore_simple_table(&$restore, $status, &$xml, &$record, $table, $record_tag) {
    // $xml is an XML tree for a stupla record
    // $record is the newly added stupla record
    return stupla_restore_records(
        $restore, $status, $xml, $table, array('stupla'=>$record->id, 'userid' => 'user'), '', '', $record_tag
    );
}

//This function returns a log record with all the necessay transformations
//done. It's used by restore_log_module() to restore modules log.
function stupla_restore_logs($restore, $log) {
    // assume the worst
    $status = false;
    switch ($log->action) {
        case "add":
        case "update":
        case "view":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code, $log->module, $log->info);
                if ($mod) {
                    $log->url = "view.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
        break;
        case "view all":
            $log->url = "index.php?id=".$log->course;
            $status = true;
        break;
        case "report":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    $log->url = "report.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
        break;
        case "attempt":
        case "submit":
        case "review":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    //Extract the attempt id from the url field
                    $attemptid = substr(strrchr($log->url,"="),1);
                    //Get the new_id of the attempt (to recode the url field)
                    $attempt = backup_getid($restore->backup_unique_code,"stupla_attempts",$attemptid);
                    if ($attempt) {
                        $log->url = "review.php?id=".$log->cmid."&attempt=".$attempt->new_id;
                        $log->info = $mod->new_id;
                        $status = true;
                    }
                }
            }
        break;
        default:
            // Oops, unknown $log->action
            if (!defined('RESTORE_SILENTLY')) {
                print "<p>action (".$log->module."-".$log->action.") unknown. Not restored</p>";
            }
        break;
    } // end switch
    return $status ? $log : false;
}

	function stupla_safe_uncode($s) {
		if ( isset($s) ) {
			$s1 = str_replace("&x01;", "\x01", $s);
			$s1 = str_replace("&x02;", "\x02", $s1);
			$s1 = str_replace("&x05;", "\x05", $s1);
			$s1 = str_replace("&x06;", "\x06", $s1);
			$s1 = str_replace("&x08;", "\x08", $s1);
			$s1 = str_replace("&xbr;", "\n", $s1);
			return $s1;
		}
		return $s;
	}

?>